char* rChangeTopic = "MoodLight/RedChange";
char* gChangeTopic = "MoodLight/GreenChange";
char* bChangeTopic = "MoodLight/BlueChange";
char* iChangeTopic = "MoodLight/IntensityChange";
char* modeChangeTopic = "MoodLight/ModeChange";
char* changeTopics[4] = {rChangeTopic, gChangeTopic, bChangeTopic, iChangeTopic};
char* rTopic = "MoodLight/Red";
char* gTopic = "MoodLight/Green";
char* bTopic = "MoodLight/Blue";
char* iTopic = "MoodLight/Intensity";
char* modeTopic = "MoodLight/Mode";
char* mainTopics[4] = {rTopic, gTopic, bTopic, iTopic};
char* modes[] = {"Static Color", "Red Alert", "Random Color", "Blue Alert"};
char* statusTopic = "MoodLight/Status";
char* clientName = "MoodLight";

void mqttCallback(char* topic, byte* payload, uint16_t len) {
  char subBuffer[20];
  memset(subBuffer, '\0', sizeof(subBuffer));
  strncpy(subBuffer, (char*) payload, len);
  unchanged = false;
  lastActiveTime = millis();

  int index = -1;
  for (uint8_t i=0; i<sizeof(changeTopics)/sizeof(changeTopics[0]); i++) {
    if (strcmp(topic, changeTopics[i]) == 0) {
      index = i;
      break;
    }
  }

  if (index != -1) {
    lightVals[index] = atoi(subBuffer);
    publishToColor(index, lightVals[index]);
    if (currentMode != 0) {
      currentMode = 0;
    }
  }
  else if (strcmp(topic, modeChangeTopic) == 0) {
    if (strcmp(subBuffer, "Red Alert") == 0) {
      currentMode = 1;
    }
    else if (strcmp(subBuffer, "Random Color") == 0) {
      currentMode = 2;
    }
    else if (strcmp(subBuffer, "Blue Alert") == 0) {
      currentMode = 3;
    }
    else {
      currentMode = 0;
    }
  }
}

void publishToColor(uint8_t color, uint8_t num) {
  char pubBuf[3];
  itoa(num, pubBuf, 10);
  client.publish(mainTopics[color], (byte*) pubBuf, strlen(pubBuf), true);
}

void publishMode(uint8_t modeNumber) {
  client.publish(modeTopic, (byte*) modes[modeNumber], strlen(modes[modeNumber]), true);
}

void connectToWiFi() {
  WiFi.begin(SSID, PASSWORD);
  while (WiFi.status() != WL_CONNECTED);
  display.clear();
  display.drawString(0, 20, "IP: " + WiFi.localIP().toString());
  display.display();
  delay(1000);
  display.clear();
  display.display();
}

void setUpMQTT() {
  client.setServer(MQTTServer, 1883);
  client.setCallback(mqttCallback);
  client.connect(clientName, statusTopic, 0, true, "Offline");
  for (int i=0; i<4; i++) {
    client.subscribe(changeTopics[i]);
    char lightValStr[3];
    itoa(lightVals[i], lightValStr, 10);
    client.publish(mainTopics[i], (byte*) lightValStr, strlen(lightValStr), true);
  }
  client.subscribe(modeChangeTopic);
  client.publish(statusTopic, (byte*) "Online", strlen("Online"), true);
}

void checkConnectionReconnect() {
  if (!client.connected()) {
    connectToWiFi();
    setUpMQTT();
  }
}